/*
 * Copyright 2004-2005 Trypticon
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.trypticon.snipsnap.macro;

import java.io.Writer;
import java.io.IOException;

import org.trypticon.snipsnap.bot.BotComponent;

import org.radeox.macro.BaseMacro;
import org.radeox.macro.parameter.MacroParameter;
import org.radeox.util.Encoder;
import org.snipsnap.container.Components;
import org.jabberstudio.jso.Presence;
import org.jabberstudio.jso.JID;
import org.apache.commons.lang.StringUtils;

/**
 * Macro which will display an XMPP link or status.
 */
public class XmppMacro extends BaseMacro
{
    /**
     * A reference to the bot.
     */
    private BotComponent xmppBot;

    /**
     * Gets the shared reference to the bot.
     *
     * @return the reference to the box.
     */
    private BotComponent getXmppBot()
    {
        if (null == xmppBot)
        {
            xmppBot = (BotComponent) Components.getComponent(BotComponent.class);
        }
        return xmppBot;
    }

    /**
     * Executes the macro.
     *
     * @param writer the writer to append to.
     * @param params the parameters passed to the macro.
     * @throws IOException if there was an error appending to the writer.
     */
    public void execute(Writer writer, MacroParameter params)
        throws IOException
    {
        if (params.getLength() < 1)
        {
            throw new IllegalArgumentException("Missing parameter 1: Jabber ID");
        }

        String jidString = params.get(0);
        JID jid = JID.valueOf(jidString);

        // Determine the status of the user.
        String fullStatus;
        Presence presence = getXmppBot().getRoster().getPresence(jid);
        if (presence == null)
        {
            fullStatus = "unavailable";
        }
        else
        {
            fullStatus = presence.getShow();
            if (StringUtils.isEmpty(fullStatus))
            {
                fullStatus = "available";
            }

            String status = presence.getStatus();
            if (status != null && status.trim().length() > 0)
            {
                fullStatus = fullStatus + ": " + status.trim();
            }
        }

        // Sanitise the strings before output!
        jidString = Encoder.escape(jidString);
        fullStatus = Encoder.escape(fullStatus);

        // Display flags.
        boolean showLink = params.get("link") == null ? true : params.get("link").equals("true");
        boolean showStatus = params.get("status") == null ? false : params.get("status").equals("true");

        // Output what was desired.
        if (showLink)
        {
            writer.write("<a href=\"xmpp:" + jidString + "\" title=\"" + fullStatus + "\">" + jidString + "</a>");
        }
        if (showLink && showStatus)
        {
            writer.write(" (");
        }
        if (showStatus)
        {
            writer.write(fullStatus);
        }
        if (showLink && showStatus)
        {
            writer.write(")");
        }
    }

    /**
     * Gets the name of the macro, which becomes the macro's prefix.
     *
     * @return the name of the macro.
     */
    public String getName()
    {
        return "xmpp";
    }

    /**
     * Gets the description of the macro.
     *
     * @return the description of the macro.
     */
    public String getDescription()
    {
        return "Formats a Jabber ID as an xmpp: hyperlink or status string.";
    }

    /**
     * Gets the description of the macro's parameters.
     *
     * @return the description of the macro's parameters, each parameters in an element of the array.
     */
    public String[] getParamDescription()
    {
        String[] description =
        {
            "1: The Jabber ID to show",
            "link: Whether to display the link (true/false, defaults to true)",
            "status: Whether to display the status string (true/false, defaults to false)"
        };
        return description;
    }
}
